from gisaid.helpers import *
from gisaid.auth import *
import time


class GiSaid(object):
    """
    Class for uploading & downloading to GISAID.
    Provides a route for automation or back-end integration.

    Parameters
    ----------
    args:
        csv_path, fasta_path, jsoncred_path or authentication info


    Returns
    ----------
    response:
        output from request


    Examples
    ----------
    >>> gs = GiSaid(authenticate=True, client_id=client_id,
    >>>              username=username, password=password, filename=filename)
    Authentication successful
    """

    def __init__(self, *args, **kwargs):
        if not kwargs:
            self.kwargs = None
            self.args = args
            self.data = read_files(self.args)
            self.authf = authfile()
        elif kwargs["authenticate"]:
            self.kwargs = kwargs
            self.args = None
            self.data = authenticate(self.kwargs)
        elif kwargs["collate_fasta"]:
            self.kwargs = kwargs
            self.args = args
            self.data = collate_fa(self)
        else:
            if kwargs["authenticate"]:
                self.kwargs = kwargs
                self.args = None
                self.data = authenticate(self.kwargs)
            else:
                print("Invalid parameter")

    def upload(self):
        """
        Uploading method

        Parameters
        ----------
        self:
            csv_path, fasta_path, jsoncred_path


        Returns
        ----------
        response:
            output from request


        Examples
        ----------
        >>> gs = GiSaid(csv_file, fasta_file, jsoncred_file)
        >>> gs.upload()
        Upload successful
        """

        s = requests.Session()
        urls = "https://gpsapi.epicov.org/epi3/gps_api"
        resp1 = s.post(
            url=urls,
            data=json.dumps(
                {
                    "cmd": "state/session/logon",
                    "api": {"version": 1},
                    "ctx": "CoV",
                    "client_id": self.authf["client_id"],
                    "auth_token": self.authf["client_token"],
                }
            ),
        )

        time.sleep(0.01)
        try:
            resp2 = [
                logfile(
                    x["covv_virus_name"],
                    s.post(
                        url=urls,
                        data=json.dumps(
                            {
                                "cmd": "data/hcov-19/upload",
                                "sid": resp1.json()["sid"],
                                "data": x,
                                "submitter": x["submitter"],
                            }
                        ),
                    ).json(),
                )
                for x in self.data
            ]
        except TypeError:
            raise Exception(
                "DataError: missing or corrupt data. Reference the upload examples."
            )
        resp3 = s.post(url=urls, data=json.dumps({"cmd": "state/session/logoff"}))
        count = 0
        bad = 0
        with open("logfile.csv") as f:
            for line in f:
                if "success" in line:
                    count += 1
                else:
                    bad += 1
        print([i for i in resp2])
        print(resp3.json())
        print(f"{bad} failed uploads")
        print(f"{count} successful uploads")



import pandas as pd
from Bio import SeqIO
import glob
import re


def logfile(*args):
    if args[1]["rc"] != "ok":
        with open("logfile.csv", "a+") as f:
            line = str("\n" + f'{args[0]}, {args[1]["validation"]}')
            f.write(line)
    else:
        with open("logfile.csv", "a+") as f:
            line = str("\n" + f"{args[0]}, success")
            f.write(line)


def check_file(fname):
    d = {}
    if "collated" in fname[0]:
        d.update(collated=True)
    else:
        d.update(collated=False)
    try:
        for i in fname[0]:
            if re.search("\.csv$", i, flags=re.IGNORECASE):
                d["csv"] = i
            elif re.search("\.fa$", i, flags=re.IGNORECASE):
                d["fa"] = i
            else:
                pass
    except IndexError:
        raise
    return d


def read_files(args):
    try:
        data = check_file(args)
    except IndexError:
        raise Exception("FileError: reference your files.")
    if data["collated"]:

        x = pd.read_csv(data["csv"], index_col=0)
        x.rename({"CollectionDate": "covv_collection_date"}, axis=1, inplace=True)
        x["covv_collection_date"] = pd.to_datetime(x["covv_collection_date"])
        x["covv_collection_date"] = x["covv_collection_date"].astype(str)
        x = (x.fillna("")).replace("NaT", "")
        metadata = x.apply(lambda x: x.to_dict(), axis=1)
    else:
        try:
            seq = {k.id: str(k.seq) for k in SeqIO.parse(data["fa"], "fasta")}

            df = pd.read_csv(data["csv"])
            df["covv_collection_date"] = pd.to_datetime(df["covv_collection_date"])
            df["covv_collection_date"] = df["covv_collection_date"].astype(str)

            metadata = df.apply(lambda x: x.to_dict(), axis=1)
            # noinspection PyStatementEffect
            {
                i.update({"covv_sequence": seq[k] for k in i.values() if k in seq})
                for i in metadata
            }
        except KeyError:
            print(f'{data["error"]} not found')
        except TypeError:
            print("file not found")
    return metadata


def collate_fa(kwargs):
    if kwargs["sub"]:
        for file in glob.iglob("{}/**/*".format(kwargs["fa"])):
            with open("collated_fasta", "a+") as f:
                seq = {k.id: str(k.seq) for k in SeqIO.parse(file, "fasta")}
                f.write("\n" + seq)
    else:
        for file in glob.iglob("{}/*".format(kwargs["fa"])):
            with open("collated_fasta.fa", "a+") as f:
                seq = {k.id: str(k.seq) for k in SeqIO.parse(file, "fasta")}
                f.write("\n" + seq)
    seq = {k.id: str(k.seq) for k in SeqIO.parse("collated_fasta.fa", "fasta")}

    df = pd.read_csv(kwargs["csv"])
    df["covv_collection_date"] = pd.to_datetime(df["covv_collection_date"])
    df["covv_collection_date"] = df["covv_collection_date"].astype(str)

    metadata = df.apply(lambda x: x.to_dict(), axis=1)

    {
        i.update({"covv_sequence": seq[k] for k in i.values() if k in seq})
        for i in metadata
    }
    return metadata



from setuptools import setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name="gisaid",
    url="https://github.com/greysonlalonde/gisaid-uploader",
    download_url="https://github.com/greysonlalonde/gisaid-uploader/archive/1.0.3b0.tar.gz",
    author="Greyson R. LaLonde",
    author_email="greyson.r.lalonde@wmich.edu",
    packages=["gisaid"],
    install_requires=["pandas", "numpy", "biopython", "requests"],
    version="1.0.3b0",
    license="MIT",
    description='Simplified & efficient GISAID interactions.',
    long_description=long_description,
    long_description_content_type='text/markdown'

)



import hashlib
import json
import requests
import secrets
from configparser import ConfigParser, NoSectionError


def authfile():
    try:
        config = ConfigParser()
        config.read(r"config.py")
        auth_path = config.get("FILES", "AUTH_FILE")
    except NoSectionError:
        raise Exception("ConfigFileError: reference your config file.")
    with open(auth_path, "r") as f:
        authf = json.loads(f.read())
    return authf


def authenticate(kwargs):
    """
    kwargs = 'authenticate=True,password=password,
    client_id=client_id, user=user,file=file'
    """

    password = kwargs["password"].encode("utf8")

    salt = secrets.token_urlsafe(64)
    hashed = (
        salt
        + "/"
        + sha512_hexdigest((salt + sha512_hexdigest(password)).encode("ascii"))
    )

    resp = requests.post(
        url="https://gpsapi.epicov.org/epi3/gps_api",
        json={
            "cmd": "state/auth/get_token",
            "api": {"version": 1},
            "ctx": "CoV",
            "client_id": kwargs["client_id"],
            "login": kwargs["username"],
            "hash": hashed,
        },
    )
    if resp.json()["rc"] == "ok":
        with open(kwargs["file"], "w") as f:
            content = {
                "api": {"version": 1},
                "ctx": "CoV",
                "client_id": kwargs["client_id"],
                "client_token": resp.json()["auth_token"],
            }
            json.dump(content, f)
        config = ConfigParser()
        config.read(r"config.py")
        config.add_section("FILES")
        config.set("FILES", "AUTH_FILE", kwargs["file"])

        with open(r"config.py", "w") as f:
            config.write(f)
    else:
        resp = f'Authentication failed: {resp.json()["rc"]}'
    return print(resp.json())


def sha512_hexdigest(inp):
    hasher = hashlib.sha512()
    hasher.update(inp)
    return hasher.hexdigest()

